function all_features = get_features(header,recordings,segment,HSMM,do_suppress) %get_ECGLeads_features

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Purpose: Extract features from PCG recordings
% Inputs:
% 1. Header data
% 2. The recordings
%
% Outputs:
% features
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    age_group=get_age(header);
    
    if strcmpi(age_group,'neonate')
        age=0.5;
    elseif strcmpi(age_group,'infant')
        age=6;
    elseif strcmpi(age_group,'child')
        age=6*12;
    elseif strcmpi(age_group,'adolescent')
        age=15*12;
    elseif strcmpi(age_group,'young adult')
        age=20*12;
    else
        age=0;
    end
    
    sex_group=get_sex(header);
    if strcmpi(sex_group,'male')
        sex=0;
    elseif strcmpi(sex_group,'female')
        sex=1;
    else
        sex=2;
    end
    
    height=get_height(header);
    weight=get_weight(header);
    
    recording_locations={'AV','MV','PV','TV'};
    num_recording_locations=length(recording_locations);
    
    recording_features=nan([num_recording_locations,129]);
    
    locations=get_locations(header);
    num_locations=length(locations);
    
    for j=1:num_locations
        features = [];
        ind=strmatch(locations{j},recording_locations);

        if isempty(ind)
            continue
        end

        [segm_recording,downsample_recording] = preproc_signal(recordings{j},1000);
        
        if do_suppress
            supr_recording = suppress_breathing(downsample_recording,250,0.7,false);
        end

        if isempty(segment)          
            [s1_start,s1_stop,s2_start,s2_stop,sys_start,sys_stop,dia_start,dia_stop]=...
                segment_HSMM(HSMM.B,HSMM.pi_v,HSMM.obs,segm_recording,1000);
        else
            seg = segment{j};
            if any(cellfun('isempty',seg))
                continue
            end
            s1_start_p = seg{1}(:,1);
            s1_stop_p = seg{1}(:,2);
            s2_start_p = seg{2}(:,1);
            s2_stop_p = seg{2}(:,2);
            sys_start_p = seg{3}(:,1);
            sys_stop_p = seg{3}(:,2);
            dia_start_p = seg{4}(:,1);
            dia_stop_p = seg{4}(:,2);

            s1_start = s1_start_p(s1_start_p>1000 & s1_start_p<length(downsample_recording) & s1_stop_p<length(downsample_recording));
            s1_stop = s1_stop_p(s1_start_p>1000 & s1_start_p<length(downsample_recording) & s1_stop_p<length(downsample_recording));
            s2_start = s2_start_p(s2_start_p>1000 & s2_start_p<length(downsample_recording) & s2_stop_p<length(downsample_recording));
            s2_stop = s2_stop_p(s2_start_p>1000 & s2_start_p<length(downsample_recording) & s2_stop_p<length(downsample_recording));
            sys_start = sys_start_p(sys_start_p>1000 & sys_start_p<length(downsample_recording) & sys_stop_p<length(downsample_recording));
            sys_stop = sys_stop_p(sys_start_p>1000 & sys_start_p<length(downsample_recording) & sys_stop_p<length(downsample_recording));
            dia_start = dia_start_p(dia_start_p>1000 & dia_start_p<length(downsample_recording) & dia_stop_p<length(downsample_recording));
            dia_stop = dia_stop_p(dia_start_p>1000 & dia_start_p<length(downsample_recording) & dia_stop_p<length(downsample_recording));
        end

        [c,l] = wavedec(downsample_recording,2,'db6');
        det_1 = detcoef(c,l,1);
        det_1 = resample(det_1,length(downsample_recording),length(det_1));
        det_1 = det_1/max(det_1);

        [s1_len,K_s1,rms_s1,F_s1,~,~,zcr_s1,wt_s1,hi_lo_s1,hi_all_s1,freq_kurt_s1] = get_segment_feature(supr_recording,det_1,s1_start,s1_stop);
        [s2_len,K_s2,rms_s2,F_s2,~,~,zcr_s2,wt_s2,hi_lo_s2,hi_all_s2,freq_kurt_s2] = get_segment_feature(supr_recording,det_1,s2_start,s2_stop);
        [sys_len,K_sys,rms_sys,F_sys,~,~,zcr_sys,wt_sys,hi_lo_sys,hi_all_sys,freq_kurt_sys] = get_segment_feature(supr_recording,det_1,sys_start,sys_stop);
        [dia_len,K_dia,rms_dia,F_dia,~,~,zcr_dia,wt_dia,hi_lo_dia,hi_all_dia,freq_kurt_dia] = get_segment_feature(supr_recording,det_1,dia_start,dia_stop);

        len_s1_mean = trimmean(s1_len,5);
        len_s2_mean = trimmean(s2_len,5);
        len_s1_std = std(s1_len);
        len_s2_std = std(s2_len);
        len_s_features = [len_s1_mean,len_s2_mean,len_s1_std,len_s2_std];
        features = [features,len_s_features];

        len_sys_mean = trimmean(sys_len,5);
        len_dia_mean = trimmean(dia_len,5);
        len_sys_std = std(sys_len);
        len_dia_std = std(dia_len);
        len_in_features = [len_sys_mean,len_dia_mean,len_sys_std,len_dia_std];
        features = [features,len_in_features];

        % Frequency features (S1,S2,Sys,Dia)
        % mean, std at max, max freq
        tms1 = trimmean(F_s1,5,1);
        tms2 = trimmean(F_s2,5,1);
        tmsys = trimmean(F_sys,5,1);
        tmdia = trimmean(F_dia,5,1);
        freq_features = [tms1,tms2,tmsys,tmdia];
        features = [features,freq_features];

        std_freq_s1 = std(F_s1/max(F_s1(:)),0,1);
        std_freq_s2 = std(F_s2/max(F_s2(:)),0,1);
        std_freq_sys = std(F_sys/max(F_sys(:)),0,1);
        std_freq_dia = std(F_dia/max(F_dia(:)),0,1);
        
        [~,freq_m1] = max(tms1);
        [~,freq_m2] = max(tms2);
        [~,freq_ms] = max(tmsys);
        [~,freq_md] = max(tmdia);
        
        std_freq_s1_max = 0;
        std_freq_s2_max = 0;
        std_freq_sys_max = 0;
        std_freq_dia_max = 0;

        if length(std_freq_s1)>1
            std_freq_s1_max = std_freq_s1(freq_m1);
        end
        if length(std_freq_s2)>1
            std_freq_s2_max = std_freq_s2(freq_m2);
        end
        if length(std_freq_sys)>1
            std_freq_sys_max = std_freq_sys(freq_ms);
        end
        if length(std_freq_dia)>1
            std_freq_dia_max = std_freq_dia(freq_md);
        end
        freq_std_features = [std_freq_s1_max,std_freq_s2_max,std_freq_sys_max,std_freq_dia_max];
        features = [features,freq_std_features];
        
        freq_m1 = freq_m1*500/64;
        freq_m2 = freq_m2*500/64;
        freq_ms = freq_ms*500/64;
        freq_md = freq_md*500/64;
        freq_m_features = [freq_m1,freq_m2,freq_ms,freq_md];
        features = [features,freq_m_features];
        
        % kurtosis and rms (Sys,Dia)
        % kurt. mean and std
        kurt_sys_mean = mean(K_sys,'omitnan');
        kurt_sys_std = std(K_sys,'omitnan');
        kurt_dia_mean = mean(K_dia,'omitnan');
        kurt_dia_std = std(K_dia,'omitnan');
        kurt_s1_mean = mean(K_s1,'omitnan');
        kurt_s1_std = std(K_s1,'omitnan');
        kurt_s2_mean = mean(K_s2,'omitnan');
        kurt_s2_std = std(K_s2,'omitnan');

        % spectral kurtosis (Sys,Dia)
        freq_kurt_sys_m = trimmean(freq_kurt_sys,5);
        freq_kurt_dia_m = trimmean(freq_kurt_dia,5);
        freq_kurt_s1_m = trimmean(freq_kurt_s1,5);
        freq_kurt_s2_m = trimmean(freq_kurt_s2,5);

        kurt_features = [kurt_sys_mean,kurt_sys_std,kurt_dia_mean,kurt_dia_std,freq_kurt_sys_m,freq_kurt_dia_m];
        kurt_features(isnan(kurt_features))=0;
        features = [features,kurt_features];

        kurt_features = [kurt_s1_mean,kurt_s1_std,kurt_s2_mean,kurt_s2_std,freq_kurt_s1_m,freq_kurt_s2_m];
        kurt_features(isnan(kurt_features))=0;
        features = [features,kurt_features];

        % rms mean and std
        rms_sys_mean = mean(rms_sys,'omitnan');
        rms_sys_std = std(rms_sys,'omitnan');
        rms_dia_mean = mean(rms_dia,'omitnan');
        rms_dia_std = std(rms_dia,'omitnan');
        rms_features = [rms_sys_mean,rms_sys_std,rms_dia_mean,rms_dia_std];
        rms_features(isnan(rms_features))=0;
        features = [features,rms_features];

        rms_s1_mean = mean(rms_s1,'omitnan');
        rms_s1_std = std(rms_s1,'omitnan');
        rms_s2_mean = mean(rms_s2,'omitnan');
        rms_s2_std = std(rms_s2,'omitnan');
        rms_features = [rms_s1_mean,rms_s1_std,rms_s2_mean,rms_s2_std];
        rms_features(isnan(rms_features))=0;
        features = [features,rms_features];

%         % lyapunov exponent mean std
%         lyap_s1_mean = trimmean(lyap_s1,10);
%         lyap_s2_mean = trimmean(lyap_s2,10);
%         lyap_sys_mean = trimmean(lyap_sys,10);
%         lyap_dia_mean = trimmean(lyap_dia,10);
%         lyap_features = [lyap_s1_mean,lyap_s2_mean,lyap_sys_mean,lyap_dia_mean];
%         lyap_features(isnan(lyap_features))=0;
%         features = [features,lyap_features];
% 
%         lyap_s1_std = std(lyap_s1);
%         lyap_s2_std = std(lyap_s2);
%         lyap_sys_std = std(lyap_sys);
%         lyap_dia_std = std(lyap_dia);
%         lyap_features = [lyap_s1_std,lyap_s2_std,lyap_sys_std,lyap_dia_std];
%         lyap_features(isnan(lyap_features))=0;
%         features = [features,lyap_features];
% 
%         % lyapunov exponent on 1 wavedec mean std
%         lyap_wt_s1_mean = trimmean(wt_lyap_s1,5);
%         lyap_wt_s2_mean = trimmean(wt_lyap_s2,5);
%         lyap_wt_sys_mean = trimmean(wt_lyap_sys,5);
%         lyap_wt_dia_mean = trimmean(wt_lyap_dia,5);
%         lyap_features = [lyap_wt_s1_mean,lyap_wt_s2_mean,lyap_wt_sys_mean,lyap_wt_dia_mean];
%         lyap_features(isnan(lyap_features))=0;
%         features = [features,lyap_features];
% 
%         lyap_wt_s1_std = std(wt_lyap_s1);
%         lyap_wt_s2_std = std(wt_lyap_s2);
%         lyap_wt_sys_std = std(wt_lyap_sys);
%         lyap_wt_dia_std = std(wt_lyap_dia);
%         lyap_features = [lyap_wt_s1_std,lyap_wt_s2_std,lyap_wt_sys_std,lyap_wt_dia_std];
%         lyap_features(isnan(lyap_features))=0;
%         features = [features,lyap_features];

        % zerocrossrate mean std
        zcr_s1_mean = trimmean(zcr_s1,5);
        zcr_s2_mean = trimmean(zcr_s2,5);
        zcr_sys_mean = trimmean(zcr_sys,5);
        zcr_dia_mean = trimmean(zcr_dia,5);
        zcr_features = [zcr_s1_mean,zcr_s2_mean,zcr_sys_mean,zcr_dia_mean];
        zcr_features(isnan(zcr_features))=0;
        features = [features,zcr_features];

        zcr_s1_std = std(zcr_s1);
        zcr_s2_std = std(zcr_s2);
        zcr_sys_std = std(zcr_sys);
        zcr_dia_std = std(zcr_dia);
        zcr_features = [zcr_s1_std,zcr_s2_std,zcr_sys_std,zcr_dia_std];
        zcr_features(isnan(zcr_features))=0;
        features = [features,zcr_features];

        % wavelet detail 1 energy mean std
        wt1_s1_mean = trimmean(wt_s1,5);
        wt1_s2_mean = trimmean(wt_s2,5);
        wt1_sys_mean = trimmean(wt_sys,5);
        wt1_dia_mean = trimmean(wt_dia,5);
        wt1_features = [wt1_s1_mean,wt1_s2_mean,wt1_sys_mean,wt1_dia_mean];
        wt1_features(isnan(wt1_features))=0;
        features = [features,wt1_features];

        wt1_s1_std = std(wt_s1);
        wt1_s2_std = std(wt_s2);
        wt1_sys_std = std(wt_sys);
        wt1_dia_std = std(wt_dia);
        wt1_features = [wt1_s1_std,wt1_s2_std,wt1_sys_std,wt1_dia_std];
        wt1_features(isnan(wt1_features))=0;
        features = [features,wt1_features];

        % hi lo freq ratio mean std
        hi_lo_s1_mean = trimmean(hi_lo_s1,5);
        hi_lo_s2_mean = trimmean(hi_lo_s2,5);
        hi_lo_sys_mean = trimmean(hi_lo_sys,5);
        hi_lo_dia_mean = trimmean(hi_lo_dia,5);
        freq_ratio_features = [hi_lo_s1_mean,hi_lo_s2_mean,hi_lo_sys_mean,hi_lo_dia_mean];
        freq_ratio_features(isnan(freq_ratio_features))=0;
        features = [features,freq_ratio_features];

        hi_lo_s1_std = std(hi_lo_s1);
        hi_lo_s2_std = std(hi_lo_s2);
        hi_lo_sys_std = std(hi_lo_sys);
        hi_lo_dia_std = std(hi_lo_dia);
        freq_ratio_features = [hi_lo_s1_std,hi_lo_s2_std,hi_lo_sys_std,hi_lo_dia_std];
        freq_ratio_features(isnan(freq_ratio_features))=0;
        features = [features,freq_ratio_features];

        % hi all freq ratio mean std
        hi_all_s1_mean = trimmean(hi_all_s1,5);
        hi_all_s2_mean = trimmean(hi_all_s2,5);
        hi_all_sys_mean = trimmean(hi_all_sys,5);
        hi_all_dia_mean = trimmean(hi_all_dia,5);
        freq_ratio_features = [hi_all_s1_mean,hi_all_s2_mean,hi_all_sys_mean,hi_all_dia_mean];
        freq_ratio_features(isnan(freq_ratio_features))=0;
        features = [features,freq_ratio_features];

        hi_all_s1_std = std(hi_all_s1);
        hi_all_s2_std = std(hi_all_s2);
        hi_all_sys_std = std(hi_all_sys);
        hi_all_dia_std = std(hi_all_dia);
        freq_ratio_features = [hi_all_s1_std,hi_all_s2_std,hi_all_sys_std,hi_all_dia_std];
        freq_ratio_features(isnan(freq_ratio_features))=0;
        features = [features,freq_ratio_features];
        
        % total signal features
        sm_env = mean(abs(stft(supr_recording,1000,"Window",hann(30),"OverlapLength",29)));
        [aut,lags]=xcorr(sm_env,3000,'normalized');
        peak = max(aut(lags>300 & lags<2500));
        total_kurt = kurtosis(supr_recording);
        [psd,f] = pwelch(supr_recording,[],[],[],1000);
        lo_freq_ratio = sum(psd(f>24 & f<144))/sum(psd);
        mi_freq_ratio = sum(psd(f>144 & f<200))/sum(psd);
        hi_freq_ratio = sum(psd(f>200))/sum(psd);
        total_features = [peak,total_kurt,lo_freq_ratio,mi_freq_ratio,hi_freq_ratio];
        features = [features,total_features];

        recording_features(ind,:) = features;
    end
    
%     recording_features=recording_features(:);
    
    all_features=[mean(recording_features,'omitnan'),...
                  median(recording_features,'omitnan'),...
                  std(recording_features,'omitnan'),...
                  max(recording_features),...
                  min(recording_features),...
                  age sex height weight];
end

function age_group=get_age(header)
    age_group=header(startsWith(header,'#Age'));
    age_group=strsplit(age_group{1},':');
    age_group=strtrim(age_group{2});
end

function sex=get_sex(header)
    sex=header(startsWith(header,'#Sex'));
    sex=strsplit(sex{1},':');
    sex=strtrim(sex{2});
end

function height=get_height(header)
    height=header(startsWith(header,'#Height'));
    height=strsplit(height{1},':');
    height=strtrim(height{2});
    height=str2double(height);
    
    if isnan(height)
        height=0;
    end
end

function weight=get_weight(header)
    weight=header(startsWith(header,'#Weight'));
    weight=strsplit(weight{1},':');
    weight=strtrim(weight{2});
    weight=str2double(weight);
    
    if isnan(weight)
        weight=0;
    end
end

function locations=get_locations(header)
    num_locations=strsplit(header{1},' ');
    num_locations=str2double(num_locations{2});
    
    locations={};
    
    for j=2:num_locations+1
        current_line=strsplit(header{j},' ');
        locations{j-1}=current_line{1};
    end
end
