function [s_start,s_stop] = get_sound_events(s_locs,r_ms,win_size,start_perc_th,stop_perc_th)
%GET_SOUND_EVENTS Get start and stop events for a heartsound
%   Returned values:
%       - start events
%       - stop events
%   Input parameters:
%       - heartsound locations
%       - rms envelope
%       - search window size
%       - start event threshold (percent of rms at detected location)
%       - stop ecent threshold (percent of rms at detected location)

    s_start=[];
    s_stop=[];

    for k=1:length(s_locs)
        pos = s_locs(k);
        val = r_ms(pos);
        st = pos-round(win_size/2);
        en = pos+round(win_size/2);
        if st<1
            st=1;
        end
        if en>length(r_ms)
            en=length(r_ms);
        end
        win = r_ms(st:en);
        st_th = val*start_perc_th;
        en_th = val*stop_perc_th;
    
        st_ev = find(win>st_th,1);
        en_ev = find(win<en_th,1);
        if ~isempty(st_ev) && ~isempty(en_ev) && (st_ev+st)<length(r_ms) && (en_ev+en)<length(r_ms)
            s_start = [s_start, st_ev+st];
            s_stop = [s_stop, en_ev+en];
        end
    end
end

